@@ -32,7 +32,8 @@ revision = 'b6a2519ab7dc'
 down_revision = '2cb4ee992b41'
 
 # milestone identifier, used by neutron-db-manage
-neutron_milestone = [migration.NEWTON]
+neutron_milestone = [migration.NEWTON, migration.OCATA,
+                     migration.PIKE, migration.QUEENS]
 
 
 _INSPECTOR = None
@@ -111,7 +112,7 @@ def recreate_index(index, table_name):
     old_name = index['name']
     new_name = old_name.replace('tenant', 'project')
 
-    op.drop_index(op.f(old_name), table_name)
+    op.drop_index(index_name=op.f(old_name), table_name=table_name)
     op.create_index(new_name, table_name, ['project_id'])
 
 
