@@ -72,14 +72,14 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
                     raise vpn_exception.IPsecSiteConnectionNotFound(
                         ipsec_site_conn_id=v_id
                     )
-                elif issubclass(model, vpn_models.IKEPolicy):
+                if issubclass(model, vpn_models.IKEPolicy):
                     raise vpn_exception.IKEPolicyNotFound(ikepolicy_id=v_id)
-                elif issubclass(model, vpn_models.IPsecPolicy):
+                if issubclass(model, vpn_models.IPsecPolicy):
                     raise vpn_exception.IPsecPolicyNotFound(
                         ipsecpolicy_id=v_id)
-                elif issubclass(model, vpn_models.VPNService):
+                if issubclass(model, vpn_models.VPNService):
                     raise vpn_exception.VPNServiceNotFound(vpnservice_id=v_id)
-                elif issubclass(model, vpn_models.VPNEndpointGroup):
+                if issubclass(model, vpn_models.VPNEndpointGroup):
                     raise vpn_exception.VPNEndpointGroupNotFound(
                         endpoint_group_id=v_id)
                 ctx.reraise = True
@@ -228,9 +228,9 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
             if "peer_cidrs" in ipsec_sitecon:
                 changed_peer_cidrs = True
                 old_peer_cidr_list = ipsec_site_conn_db['peer_cidrs']
-                old_peer_cidr_dict = dict(
-                    (peer_cidr['cidr'], peer_cidr)
-                    for peer_cidr in old_peer_cidr_list)
+                old_peer_cidr_dict = {
+                    peer_cidr['cidr']: peer_cidr
+                    for peer_cidr in old_peer_cidr_list}
                 new_peer_cidr_set = set(ipsec_sitecon["peer_cidrs"])
                 old_peer_cidr_set = set(old_peer_cidr_dict)
 
@@ -263,6 +263,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         return self._get_resource(
             context, vpn_models.IPsecSiteConnection, ipsec_site_conn_id)
 
+    @db_api.CONTEXT_READER
     def get_ipsec_site_connection(self, context,
                                   ipsec_site_conn_id, fields=None):
         ipsec_site_conn_db = self._get_ipsec_site_connection(
@@ -270,6 +271,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         return self._make_ipsec_site_connection_dict(
             ipsec_site_conn_db, fields)
 
+    @db_api.CONTEXT_READER
     def get_ipsec_site_connections(self, context, filters=None, fields=None):
         return model_query.get_collection(
             context, vpn_models.IPsecSiteConnection,
@@ -368,11 +370,13 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
                 context, vpn_models.IKEPolicy, ikepolicy_id)
             context.session.delete(ike_db)
 
+    @db_api.CONTEXT_READER
     def get_ikepolicy(self, context, ikepolicy_id, fields=None):
         ike_db = self._get_resource(
             context, vpn_models.IKEPolicy, ikepolicy_id)
         return self._make_ikepolicy_dict(ike_db, fields)
 
+    @db_api.CONTEXT_READER
     def get_ikepolicies(self, context, filters=None, fields=None):
         return model_query.get_collection(context, vpn_models.IKEPolicy,
                                           self._make_ikepolicy_dict,
@@ -452,11 +456,13 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
                 context, vpn_models.IPsecPolicy, ipsecpolicy_id)
             context.session.delete(ipsec_db)
 
+    @db_api.CONTEXT_READER
     def get_ipsecpolicy(self, context, ipsecpolicy_id, fields=None):
         ipsec_db = self._get_resource(
             context, vpn_models.IPsecPolicy, ipsecpolicy_id)
         return self._make_ipsecpolicy_dict(ipsec_db, fields)
 
+    @db_api.CONTEXT_READER
     def get_ipsecpolicies(self, context, filters=None, fields=None):
         return model_query.get_collection(context, vpn_models.IPsecPolicy,
                                           self._make_ipsecpolicy_dict,
@@ -505,6 +511,19 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
             vpns_db.update(vpns)
         return self._make_vpnservice_dict(vpns_db)
 
+    def set_vpnservice_status(self, context, vpnservice_id, status,
+                              updated_pending_status=False):
+        vpns = {'status': status}
+        with db_api.CONTEXT_WRITER.using(context):
+            vpns_db = self._get_resource(context, vpn_models.VPNService,
+                                         vpnservice_id)
+            if (utils.in_pending_status(vpns_db.status) and
+                    not updated_pending_status):
+                raise vpnaas.VPNStateInvalidToUpdate(
+                    id=vpnservice_id, state=vpns_db.status)
+            vpns_db.update(vpns)
+        return self._make_vpnservice_dict(vpns_db)
+
     def update_vpnservice(self, context, vpnservice_id, vpnservice):
         vpns = vpnservice['vpnservice']
         with db_api.CONTEXT_WRITER.using(context):
@@ -526,15 +545,18 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
                                          vpnservice_id)
             context.session.delete(vpns_db)
 
+    @db_api.CONTEXT_READER
     def _get_vpnservice(self, context, vpnservice_id):
         return self._get_resource(context, vpn_models.VPNService,
                                   vpnservice_id)
 
+    @db_api.CONTEXT_READER
     def get_vpnservice(self, context, vpnservice_id, fields=None):
         vpns_db = self._get_resource(context, vpn_models.VPNService,
                                      vpnservice_id)
         return self._make_vpnservice_dict(vpns_db, fields)
 
+    @db_api.CONTEXT_READER
     def get_vpnservices(self, context, filters=None, fields=None):
         return model_query.get_collection(context, vpn_models.VPNService,
                                           self._make_vpnservice_dict,
@@ -593,8 +615,8 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
             query = query.join(
                 vpn_models.VPNEndpoint,
                 sa.and_(vpn_models.VPNEndpoint.endpoint_group_id ==
-                     vpn_models.VPNEndpointGroup.id,
-                     vpn_models.VPNEndpoint.endpoint == subnet_id))
+                        vpn_models.VPNEndpointGroup.id,
+                        vpn_models.VPNEndpoint.endpoint == subnet_id))
             group = query.first()
             if group:
                 raise vpn_exception.SubnetInUseByEndpointGroup(
@@ -648,11 +670,13 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
                 context, vpn_models.VPNEndpointGroup, endpoint_group_id)
             context.session.delete(endpoint_group_db)
 
+    @db_api.CONTEXT_READER
     def get_endpoint_group(self, context, endpoint_group_id, fields=None):
         endpoint_group_db = self._get_resource(
             context, vpn_models.VPNEndpointGroup, endpoint_group_id)
         return self._make_endpoint_group_dict(endpoint_group_db, fields)
 
+    @db_api.CONTEXT_READER
     def get_endpoint_groups(self, context, filters=None, fields=None):
         return model_query.get_collection(context, vpn_models.VPNEndpointGroup,
                                           self._make_endpoint_group_dict,
@@ -673,8 +697,24 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
             vpnservice = self._get_vpnservice(context, vpnservice_id)
             return vpnservice['router_id']
 
+    @db_api.CONTEXT_READER
+    def get_peer_cidrs_for_router(self, context, router_id):
+        filters = {'router_id': [router_id]}
+        vpnservices = model_query.get_collection_query(
+            context, vpn_models.VPNService, filters=filters).all()
+        cidrs = []
+        for vpnservice in vpnservices:
+            for ipsec_site_connection in vpnservice.ipsec_site_connections:
+                if ipsec_site_connection.peer_cidrs:
+                    for peer_cidr in ipsec_site_connection.peer_cidrs:
+                        cidrs.append(peer_cidr.cidr)
+                if ipsec_site_connection.peer_ep_group is not None:
+                    for ep in ipsec_site_connection.peer_ep_group.endpoints:
+                        cidrs.append(ep.endpoint)
+        return cidrs
 
-class VPNPluginRpcDbMixin(object):
+
+class VPNPluginRpcDbMixin:
     def _build_local_subnet_cidr_map(self, context):
         """Build a dict of all local endpoint subnets, with list of CIDRs."""
         query = context.session.query(models_v2.Subnet.id,
