@@ -136,7 +136,7 @@ def _get_template(template_file):
     return JINJA_ENV.get_template(template_file)
 
 
-class BaseSwanProcess(object, metaclass=abc.ABCMeta):
+class BaseSwanProcess(metaclass=abc.ABCMeta):
     """Swan Family Process Manager
 
     This class manages start/restart/stop ipsec process.
@@ -163,8 +163,12 @@ class BaseSwanProcess(object, metaclass=abc.ABCMeta):
     DIALECT_MAP = {
         "3des": "3des",
         "aes-128": "aes128",
-        "aes-256": "aes256",
         "aes-192": "aes192",
+        "aes-256": "aes256",
+        "aes-ctr-128": "aesctr128",
+        "aes-ctr-192": "aesctr192",
+        "aes-ctr-256": "aesctr256",
+        "aes-xcbc": "aes_xcbc",
         "sha256": "sha2_256",
         "sha384": "sha2_384",
         "sha512": "sha2_512",
@@ -172,6 +176,16 @@ class BaseSwanProcess(object, metaclass=abc.ABCMeta):
         "group5": "modp1536",
         "group14": "modp2048",
         "group15": "modp3072",
+        "group16": "modp4096",
+        "group17": "modp6144",
+        "group18": "modp8192",
+        "group19": "ecp256",
+        "group20": "ecp384",
+        "group21": "ecp521",
+        "group22": "dh22",
+        "group23": "dh23",
+        "group24": "dh24",
+        "group31": "curve25519",
         "bi-directional": "start",
         "response-only": "add",
         "v2": "insist",
@@ -187,9 +201,9 @@ class BaseSwanProcess(object, metaclass=abc.ABCMeta):
     STATUS_RE = r'\d\d\d "([a-f0-9\-]+).* (unrouted|erouted);'
     STATUS_NOT_RUNNING_RE = 'Command:.*ipsec.*status.*Exit code: [1|3]$'
     STATUS_IPSEC_SA_ESTABLISHED_RE = (
-        r'\d{3} #\d+: "([a-f0-9\-]+).*established.*newest IPSEC')
+        r'\d{3} #\d+: "([a-f0-9\-]+).*established.*newest')
     STATUS_IPSEC_SA_ESTABLISHED_RE2 = (
-        r'\d{3} #\d+: "([a-f0-9\-\/x]+).*established.*newest IPSEC')
+        r'\d{3} #\d+: "([a-f0-9\-\/x]+).*established.*newest')
 
     def __init__(self, conf, process_id, vpnservice, namespace):
         self.conf = conf
@@ -452,8 +466,7 @@ class OpenSwanProcess(BaseSwanProcess):
     IPSEC_CONF_NAT_TRAVERSAL = "yes"
 
     def __init__(self, conf, process_id, vpnservice, namespace):
-        super(OpenSwanProcess, self).__init__(conf, process_id,
-                                              vpnservice, namespace)
+        super().__init__(conf, process_id, vpnservice, namespace)
         self.secrets_file = os.path.join(
             self.etc_dir, 'ipsec.secrets')
         self.config_file = os.path.join(
@@ -508,7 +521,7 @@ class OpenSwanProcess(BaseSwanProcess):
             # on throwing to tell us something. If the pid file exists,
             # delve into the process information and check if it matches
             # our expected command line.
-            with open(self.pid_file, 'r') as f:
+            with open(self.pid_file) as f:
                 pid = f.readline().strip()
                 with open('/proc/%s/cmdline' % pid) as cmd_line_file:
                     cmd_line = cmd_line_file.readline()
@@ -525,7 +538,7 @@ class OpenSwanProcess(BaseSwanProcess):
                                   {'pid': pid, 'cmd_line': cmd_line})
                         return True
 
-        except IOError as e:
+        except OSError as e:
             # This is logged as "info" instead of error because it simply
             # means that we couldn't find the files to check on them.
             LOG.info('Unable to find control files on startup for '
@@ -638,7 +651,7 @@ class OpenSwanProcess(BaseSwanProcess):
             nets += ipsec_site_conn['peer_cidrs']
         for net in nets:
             version = netaddr.IPNetwork(net).version
-            virtual_privates.append('%%v%s:%s' % (version, net))
+            virtual_privates.append('%v{}:{}'.format(version, net))
         virtual_privates.sort()
         return ','.join(virtual_privates)
 
@@ -788,7 +801,7 @@ class OpenSwanProcess(BaseSwanProcess):
         self.connection_status = {}
 
 
-class IPsecVpnDriverApi(object):
+class IPsecVpnDriverApi:
     """IPSecVpnDriver RPC api."""
 
     @log_helpers.log_method_call
@@ -838,7 +851,7 @@ class IPsecDriver(device_drivers.DeviceDriver, metaclass=abc.ABCMeta):
         self.conn = n_rpc.Connection()
         self.context = context.get_admin_context_without_session()
         self.topic = topics.IPSEC_AGENT_TOPIC
-        node_topic = '%s.%s' % (self.topic, self.host)
+        node_topic = '{}.{}'.format(self.topic, self.host)
 
         self.processes = {}
         self.routers: ty.Dict[str, ty.Any] = {}
